package com.jxy.system.api;

import com.jxy.common.core.Result.R;
import com.jxy.common.core.constant.SecurityConstants;
import com.jxy.common.core.constant.ServiceNameConstants;
import com.jxy.system.api.entity.SysUser;
import com.jxy.system.api.entity.SysUserLeveDate;
import com.jxy.system.api.factory.RemoteUserFallbackFactory;
import com.jxy.system.api.model.LoginUser;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

/**
 * @Description 用户服务
 * @Date 2022/5/12 15:46
 * @Author 杜懿
 */
@FeignClient(contextId = "remoteUserService",value = ServiceNameConstants.SYSTEM_SERVICE,fallbackFactory = RemoteUserFallbackFactory.class)
public interface RemoteUserService {

    /**
     * 通过用户名/邮箱查询用户信息
     *
     * @param account 用户名/邮箱
     * @param source 请求来源
     * @return 结果
     */
    @GetMapping("/user/info/{account}")
    public R<LoginUser> getUserInfo(@PathVariable("account") String account, @RequestHeader(SecurityConstants.FROM_SOURCE) String source);

    /**
     * 修改用户最后一次登录的ip和登录次数
     *
     * @param sysUser 用户信息
     * @return 结果
     */
    @PostMapping("/user/update")
    public R<Boolean> updateUserInfo(@RequestBody SysUser sysUser);

    /**
     * 注册用户信息
     *
     *
     * @param sysUser 用户信息
     * @return 结果
     */
    @PostMapping("/user/register")
    public R<Boolean> registerUserInfo(@RequestBody SysUser sysUser);

    /**
     * 重置用户密码
     *
     * @param sysUser 用户信息
     * @return 结果
     */
    @PostMapping("/user/profile/resetPwd")
    public R<Boolean> resetPwdByEmail(@RequestBody SysUser sysUser);

    /**
     * 查看会员有效期情况
     * @return 结果
     */
    @PostMapping("/user/profile/userLevel")
    public R<SysUserLeveDate> userLevel(@RequestBody Long userId);

    /**
     * 升级用户并记录用户会员有效期
     *
     * @param user 用户信息
     * @return 结果
     */
    @PostMapping("/user/profile/Levelup")
    public R<Boolean> userLevelup(@RequestBody SysUserLeveDate user, @RequestHeader(SecurityConstants.FROM_SOURCE) String source);


}
