package com.jxy.common.security.feign;

import com.jxy.common.core.constant.SecurityConstants;
import com.jxy.common.core.utils.ServletUtils;
import com.jxy.common.core.utils.StringUtils;
import com.jxy.common.core.utils.ip.IpUtils;
import feign.RequestInterceptor;
import feign.RequestTemplate;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * @Description fegin 请求拦截器
 * @Date 2022/5/13 11:12
 * @Author 杜懿
 */
@Component
public class FeignRequestInterceptor implements RequestInterceptor {
    @Override
    public void apply(RequestTemplate template) {
        System.out.println("feign拦截器开始执行");
        HttpServletRequest httpServletRequest = ServletUtils.getRequest();
        if (StringUtils.isNotNull(httpServletRequest))
        {
            Map<String, String> headers = ServletUtils.getHeaders(httpServletRequest);
            // 传递用户信息请求头，防止丢失
            String userId = headers.get(SecurityConstants.DETAILS_USER_ID);
            if (StringUtils.isNotEmpty(userId))
            {
                template.header(SecurityConstants.DETAILS_USER_ID, userId);
            }
            String userName = headers.get(SecurityConstants.DETAILS_USERNAME);
            if (StringUtils.isNotEmpty(userName))
            {
                template.header(SecurityConstants.DETAILS_USERNAME, userName);
            }
            String authentication = headers.get(SecurityConstants.AUTHORIZATION_HEADER);
            if (StringUtils.isNotEmpty(authentication))
            {
                template.header(SecurityConstants.AUTHORIZATION_HEADER, authentication);
            }
            System.out.println("userId:"+userId);
            System.out.println("userName:"+userName);
            System.out.println("ip:"+IpUtils.getIpAddr(ServletUtils.getRequest()));
            // 配置客户端IP
            template.header("X-Forwarded-For", IpUtils.getIpAddr(ServletUtils.getRequest()));
        }
    }
}
