package com.jxy.common.security.interceptor;

import com.jxy.common.core.constant.SecurityConstants;
import com.jxy.common.core.context.SecurityContextHolder;
import com.jxy.common.core.utils.ServletUtils;
import com.jxy.common.core.utils.StringUtils;
import com.jxy.common.security.auth.AuthUtil;
import com.jxy.common.security.utils.SecurityUtils;
import com.jxy.system.api.model.LoginUser;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.AsyncHandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @Description 自定义请求头拦截器，将请求头数据封装到线程变量中方便后续获取使用
 * 同时获取当前用户有效时间并刷新有效期
 * @Date 2022/5/13 17:29
 * @Author 杜懿
 */
public class HeaderInterceptor implements AsyncHandlerInterceptor {

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if(!(handler instanceof HandlerMethod)){

            return true;

        }

        SecurityContextHolder.setUserId(ServletUtils.getHeader(request, SecurityConstants.DETAILS_USER_ID));
        SecurityContextHolder.setUserName(ServletUtils.getHeader(request,SecurityConstants.DETAILS_USERNAME));
        SecurityContextHolder.setUserKey(ServletUtils.getHeader(request,SecurityConstants.USER_KEY));

        String token = SecurityUtils.getToken();
        if(StringUtils.isNotEmpty(token)){

            LoginUser loginUser = AuthUtil.getLoginUser(token);
            if(StringUtils.isNotNull(loginUser)){

                //验证当前用户有效时间
                AuthUtil.verifyLoginUserExpire(loginUser);
                SecurityContextHolder.set(SecurityConstants.LOGIN_USER,loginUser);
            }

        }
        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {

        SecurityContextHolder.remove();

    }
}
